"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const mini_css_extract_plugin_1 = tslib_1.__importDefault(require("mini-css-extract-plugin"));
const html_webpack_plugin_1 = tslib_1.__importDefault(require("html-webpack-plugin"));
const terser_webpack_plugin_1 = tslib_1.__importDefault(require("terser-webpack-plugin"));
const optimize_css_assets_webpack_plugin_1 = tslib_1.__importDefault(require("optimize-css-assets-webpack-plugin"));
const webpack_node_externals_1 = tslib_1.__importDefault(require("webpack-node-externals"));
const config_1 = require("./config");
function createWebpackConfig(isServer) {
    const config = config_1.getMergedConfig();
    const resolveAppPath = (p) => path_1.default.resolve(config.rootDir, p);
    const plugins = getPlugins(isServer, config);
    const optimization = getOptimization(isServer, config);
    const webpackConfig = {
        bail: true,
        mode: (config.isDev || isServer) ? 'development' : 'production',
        // only client and in development mode
        devtool: (config.isDev && !isServer) ? 'cheap-module-source-map' : false,
        stats: 'errors-warnings',
        target: isServer ? 'node' : 'web',
        entry: {
            app: isServer ? config.serverEntry : resolveAppPath('src/index.tsx')
        },
        output: {
            path: path_1.default.join(config.distDir, isServer ? 'server' : 'client'),
            publicPath: isServer ? '/' : config.publicPath,
            filename: (!isServer && !config.isDev) ? 'js/[name].[chunkhash:5].js' : '[name].js',
            chunkFilename: (!isServer && !config.isDev) ? 'js/[name].[chunkhash:5].chunk.js' : '[name].chunk.js',
            libraryTarget: isServer ? 'commonjs' : 'umd',
        },
        resolve: {
            extensions: ['.js', '.jsx', '.ts', '.tsx'],
            alias: {
                'server-renderer': isServer
                    ? 'server-renderer/lib/server.js'
                    : 'server-renderer/lib/client.js',
                src: resolveAppPath('src'),
            },
        },
        module: {
            rules: [
                {
                    test: /\.tsx?$/,
                    include: path_1.default.resolve(config.rootDir, 'src'),
                    loader: require.resolve('ts-loader'),
                    options: {
                        transpileOnly: true,
                    },
                },
                {
                    test: /\.s?css$/,
                    use: getStyleLoaders(isServer, true, config)
                },
            ]
        },
        externals: isServer ? [webpack_node_externals_1.default()] : undefined,
        plugins,
        optimization,
        node: {
            require: true,
        }
    };
    return webpackConfig;
}
exports.createWebpackConfig = createWebpackConfig;
function getPlugins(isServer, config) {
    const plugins = [];
    if (!isServer && !config.isDev) {
        plugins.push(new mini_css_extract_plugin_1.default({
            filename: 'style/[name].[contenthash:5].css',
            chunkFilename: 'style/[name].[contenthash:5].css',
        }), new html_webpack_plugin_1.default({
            template: config.htmlTemplatePath,
            filename: config.builtHTMLPath,
        }));
    }
    return plugins;
}
function getOptimization(isServer, config) {
    if (isServer || config.isDev) {
        return undefined;
    }
    const useSourceMap = !isServer && !config.isDev;
    return {
        minimizer: [
            new terser_webpack_plugin_1.default({
                extractComments: false,
                terserOptions: {
                    sourceMap: useSourceMap,
                    safari10: true,
                    output: {
                        comments: false,
                    }
                }
            }),
            new optimize_css_assets_webpack_plugin_1.default(),
        ],
        splitChunks: {
            chunks: 'all',
            name: true,
            cacheGroups: {
                vonders: {
                    test: /[\\/]node_modules[\\/]/,
                    name: 'vonders',
                    priority: 10,
                },
            },
        },
        runtimeChunk: {
            name: 'runtime',
        },
    };
}
function getStyleLoaders(isServer, isModule, config) {
    const useSourceMap = !isServer && !config.isDev;
    const cssLoaderWithModule = {
        loader: require.resolve('css-loader'),
        options: {
            importLoaders: 1,
            modules: isModule && {
                localIdentName: '[name]-[local]',
            },
            localsConvention: 'camelCaseOnly',
            sourceMap: useSourceMap,
        },
    };
    const styleLoader = isServer
        ? require.resolve('isomorphic-style-loader')
        : config.isDev
            ? require.resolve('style-loader')
            : mini_css_extract_plugin_1.default.loader;
    const loaders = [
        styleLoader,
        cssLoaderWithModule,
    ];
    if (!isServer && !config.isDev) {
        loaders.push({
            loader: require.resolve('postcss-loader'),
            options: {
                ident: 'postcss',
                plugins: () => [
                    require('postcss-flexbugs-fixes'),
                    require('postcss-preset-env')({
                        autoprefixer: {
                            flexbox: 'no-2009',
                        },
                        stage: 3,
                    }),
                ],
                sourceMap: useSourceMap,
            }
        });
    }
    loaders.push({
        loader: require.resolve('sass-loader'),
        options: {
            sourceMap: useSourceMap,
        }
    });
    return loaders;
}
// function getTsLoader(isServer: boolean, config: Config): webpack.RuleSetRule {
//   const srcDir = path.resolve(config.rootDir, 'src')
//   if (isServer) {
//     return {
//       test: /\.tsx?$/,
//       include: srcDir,
//       loader: require.resolve('ts-loader'),
//       options: {
//         transpileOnly: true,
//       }
//     }
//   }
//   return {
//     test: /\.tsx?$/,
//     loader: require.resolve('babel-loader'),
//     options: {
//       plugins: [
//         require.resolve('@babel/plugin-proposal-class-properties'),
//         [
//           require.resolve('@babel/plugin-transform-runtime'),
//           {
//             helpers: true,
//             regenerator: true,
//             useESModules: false
//           }
//         ]
//       ],
//       presets: [
//         require.resolve('@babel/preset-react'),
//         require.resolve('@babel/preset-env'),
//         [
//           require.resolve('@babel/preset-typescript'),
//           {
//             isTSX: true,
//             allExtensions: true,
//             allowNamespaces: false,
//           }
//         ],
//       ],
//     }
//   }
// }
